/****************************************************************************
Date: 5/16/99
File: cfg.hpp
Description:	CONFIGURATION SERVER CLASS

History:
	4/6/99 added default configuration commands
	5/16/99 added "downlink" and "downlink2" keywords
*****************************************************************************/
#ifndef CFG_HPP
#define CFG_HPP

#include <stddef.h>

#include "loctypes.h"
#include "parser.hpp"
#include "sat.hpp"
#include "radio.hpp"
#include "strtab.hpp"	// string lookup table class

//***** Configuration File Names *****************************************
// The config file has the radio and transponder configurations for the
// satellites.
// The select file has the ID of the currently selected configuration for each
// satellite. If there is no select file or no entry for a given satellite,
// the default is the first one in the config file.
const CHAR* const CONFIG_FILE 	    = "itune.cfg"; // config file
const CHAR* const CONFIG_SELECTFILE = "itune.sel"; // config ID selection file
const CHAR CONFIG_FIRST = 0; // Select default config ID

struct CONFIGINFO {
	SATELLITEPARAMS	SatParams;
	RADIOPARAMS	RxParams;	// Rx or Xcvr
	RADIOPARAMS	Rx2Params;
	RADIOPARAMS	TxParams;
};

//***** Error Processing *******************************
enum CONFIG_ERROR {
	CONFIG_ERROR_CFGFILEOPEN,	// can't open config file
	CONFIG_ERROR_TRANSLATION,	// already have uplink translation
	CONFIG_ERROR_LINK,		// already have beacon/uplink
	CONFIG_ERROR_RADIO,		// already have radio
	CONFIG_ERROR_UNKNOWNRADIO,	// unknown radio type
	CONFIG_ERROR_CONVERTER,	// already have freq converter
	CONFIG_ERROR_COMPORT,		// com out of range
	CONFIG_ERROR_NOTFOUND,		// unknown command in parse file
	CONFIG_ERROR_SOFTWARE		// bug detected!
};


class CONFIG {
private:
        //***** Configuration Record *************************
	enum {  SATNAMESIZE = 19, 	// max satellite name string size
		CONFIGIDSIZE = 5	// max size of config ID string
	};	
	CHAR		szName[SATNAMESIZE + 1];	// +1 for NUL terminator
	CHAR		szConfigID[CONFIGIDSIZE + 1];
	CONFIGINFO	Config;		// configuration record
	CONFIGINFO	DefaultConfig;	// default configuration
	CONFIGINFO*	pConfig;	// points to one of the above
	
        //***** COM PORT TABLE ********************************
        enum { MAXCOMPORT = 4 };	// max number of serial ports
        struct COMPORTINFO {
		IOADDRESS	Address;
		USHORT		unIRQ;
		USHORT		unRate;
	};
	COMPORTINFO	aComPortInfo[MAXCOMPORT];

	//***** Radio lookup table ****************
        static const CHAR*	aRadioNames[];
        STRINGTABLE		RadioTable;
        RADIOTYPE GetRadioType(const CHAR* pszName) const {
			return (RADIOTYPE) RadioTable.GetIndex(pszName);
	}

        //***** Modulation Lookup Table ************
        static const CHAR*	aModulationNames[];
        STRINGTABLE		ModulationTable;
	MODULATION GetModulation(const CHAR* pszName) const {
                	return (MODULATION) ModulationTable.GetIndex(pszName);
	}

        //***** Keyword Lookup Table ***********************************
	enum	KEYWORD {
		KEYWORD_NOTFOUND = 0,
		KEYWORD_DEFAULT,
		KEYWORD_SATELLITE,
		KEYWORD_TRANSLATION1,
		KEYWORD_TRANSLATION2,
		KEYWORD_BEACON,
		KEYWORD_DOWNLINK,
		KEYWORD_DOWNLINK2,
		KEYWORD_UPLINK,
		KEYWORD_RX,
		KEYWORD_RX2,
		KEYWORD_TX,
		KEYWORD_XCVR,
		KEYWORD_RXCONVERTER,
		KEYWORD_RXCONVERTER2,
		KEYWORD_TXCONVERTER,
		KEYWORD_COMPORT
	};
        static const CHAR*	aKeyWords[];
        STRINGTABLE		KeyWordTable;
	KEYWORD	GetKeyWord(const CHAR* pszToken) {
			return (KEYWORD) KeyWordTable.GetIndex(pszToken);
	}
	static const CHAR* const INVERT; // inverting xponder keyword


	//***** Functions for keyword processing ***********************
	// These functions read from the config file and set parts of 
	// the config record.
        VOID	SetTranslation(TRANSPONDERPARAMS& Params);
	VOID	SetLink(DOUBLE& dFrequency, MODULATION& eMode);
	VOID	SetRadio(RADIOPARAMS& Params);
        VOID	SetConverter(DOUBLE& dConverter);

	BOOL	ProcessLine(); // returns TRUE when a config record is ready

	// no copy or = operators
	CONFIG& operator=(const CONFIG&);
	CONFIG(const CONFIG&);

protected:
	PARSER	Parser;

	//***** Error Processing *******************************
	// Setting this Error flag stops config processing
	BOOL	eError;

	// If an error occurs during processing, this function will be called
	// Derived classes can catch these errors if they provide this callback
	virtual VOID Error(CONFIG_ERROR eErrorType) {
		eError = BOOL_TRUE;	// stop processing
	}

	//***** Low level utility functions *******************************
	BOOL	Find(const CHAR FAR* lpszSatName, const CHAR* pszDesiredID);
	BOOL 	Read();
	CHAR*	GetSatelliteName() { return szName; }
	CHAR*	GetConfigID() { return szConfigID; }
	BOOL	AnyErrors() { return eError; }

	// Get Freq() Gets a string from the Parser and returns Hz.
	// This function is only meaningful when the Parser is opened
	// with the config file.
	DOUBLE	GetFreq();

public:
	CONFIG();
	CONFIGINFO*	GetConfig(const CHAR FAR* lpszSatName);
};
#endif // cfg.hpp
