/****************************************************************************
Date: 1/24/97 EXPERIMENTAL
File: cmd.hpp
Description: 	COMMAND INTERPRETER 

	This file provides the definitions and types for the command
	interpreter module. 

	This module interprets commands sent via INT63 and INT64 and
	executes them by calling the appropriate member functions of
	the CONTROLLER object.

	This module uses INT64H exclusively as its RadioDRV tuning
	command interface.

	It may also (optionally) hook the OrbitDRV interrupt, INT63H
	to automatically set the radio configuration to match the current
	satellite when tracking is enabled from InstantTrack.

	The interrupt numbers for tuning commands and OrbitDRV are settable 
	using environment variables ITRAD and ITORB which take decimal 
	values. Setting ITORB to 0 prevents use of the OrbitDRV
	interrupt and disables automatic tuning control from InstantTrack.

	This module hooks the dos timer interrupt (INT1CH) to provide
	timing services needed by the CONTROLLER and INTERVALTIMER objects.

	It interprets commands sent via interrupts and executes
	them by calling the appropriate CONTROLLER object functions.

	This module include main() which loads the TSR.
	
History:

*****************************************************************************/
#ifndef CMD_HPP
#define CMD_HPP

#include "loctypes.h"
#include "debug.hpp"	// debug object allows printing to screen

//***** Environment Variables **********
const CHAR* CMD_VAR_RADIODRV = "ITRAD";	// sets radiodrv interrupt number
const CHAR* CMD_VAR_ORBITDRV = "ITORB";	// sets orbitdrv interrupt number


//***** Interrupt Numbers *********************
const INT CMD_INT_ORBITDRV   = 0x63;	// Default for OrbitDRV
const INT CMD_INT_RADIODRV   = 0x64;	// Default for RadioDRV
const INT CMD_INT_DOSTIMER   = 0x1C;	// DOS Timer interrupt


//***** Return Codes from main() *******
const INT CMD_RET_SUCCESS	= 0x300;	// TSR installed OK
const INT CMD_RET_VECTORINUSE	= 1;		// Error, vector in use 
const INT CMD_RET_NOORBITDRV	= 2;		// Error, OrbitDRV not loaded 


//***** TSR Signature ******************
// This provides the TSR signature
// that OrbitDRV, ITUNE, and ITRACK look for.
// The signature is offset from the
// interrupt vector entry point. The entry point
// provides a Long Jump to the interrupt handler 

const INT CMD_SIG_OFFSET   = 0x0A;		// signature offset
const INT CMD_SIG_SIZE     = 18;		// max signature size
#define   CMD_SIG_RADIODRV "RadioDRV"		// RadioDrv Signature
#define   CMD_SIG_ORBITDRV "RotorDRV+OrbitDRV"	// OrbitDRV signature

struct CMDSIGSTRUCT {
	BYTE	NOP;
	BYTE	LONGJUMP;
	INTHANDLER pfnIntHandler;

	BYTE	aPad[ CMD_SIG_OFFSET - (2*sizeof(BYTE)) -
					sizeof(INTHANDLER) ];
	CHAR	aSignature[CMD_SIG_SIZE];
};


//***** RADIODRV Commands **************
// sent via register AH using INT64
// The SETVELOCITY command must be 3 for compatibility
// with OrbitDRV.
enum CMD_RADIODRV {
	CMD_RADIODRV_SETVELOCITY = 3,  	// satellite velocity update
	CMD_RADIODRV_SETCONFIG,		// Set a new configuration
	CMD_RADIODRV_STOP, 		// Stop Radio Tuning
	CMD_RADIODRV_GETSTATUS,   	// Get Status
	CMD_RADIODRV_DEBUG = 0xff	// Get Debug info (developer use only)
};


//***** ORBITDRV Commands **************
// sent to ORBITDRV via register AH using INT63
// This info is from the OrbitDRV manual
enum CMD_ORBITDRV {
	CMD_ORBITDRV_SETSATELLITE = 0xfb,	// see data structure below
	CMD_ORBITDRV_SETTRACKING  = 0xfe,
	CMD_ORBITDRV_SETTUNING	  = 0xfd
};

// When SETSATELLITE command is sent to
// OrbitDRV, DS:DX points to this data structure.
struct CMDORBITDRVSATINFO {
	CHAR szName[20];		// Satellite Name
	CHAR szObject[20];		// Object id
	CHAR szNASADesignation[20];	// Nasa Designation
	DOUBLE T0, M0, N0;		// orbital params
	DOUBLE I0, E0, W0;
	DOUBLE O0;
	FLOAT F1;			// Satellite Beacon Frequency
	FLOAT N1;			
};

#endif
