/**************************************************************************
date: 8/19/96
file: fastfifo.hpp

title:    FASTFIFO - "Fast" Character FIFO class
  
public:
        FASTFIFO(UCHAR* pMemory, size_t Size)   // constructor
        VOID    Put(char c)                     // enqueue
        INT     Get()                           // dequeue
        VOID    Clear()                         // clear contents

description:
        Character FIFO applicable for use in interrupt service routines,
        drivers and wherever cpu time is critical. Buffer memory for FIFO
        is user provided. All functions are inline. Note that there is
        NO OVERRUN CHECKING on writes to the FIFO, sufficient user
        provided memory must be provided to prevent overrun.

history: initial version
**************************************************************************/
#ifndef FASTFIFO_HPP
#define FASTFIFO_HPP

#pragma check_stack( off )
#pragma check_pointer( off )

#include <stddef.h>
#include "loctypes.h"

const INT FASTFIFO_EOF = -1;	// return from Get() when empty

class FASTFIFO {
private:
	UCHAR* const pBegin;	// 1st location in user buffer
	UCHAR* const pEnd;	// points AFTER end of user buffer
        UCHAR* pRead;		// current read pointer
        UCHAR* pWrite;		// current write pointer

	//***** Get Next Pointer **********************************
        UCHAR* Next(UCHAR* p) {
                ++p;
                if (p == pEnd) {
                        p = pBegin;
                }
                return p;
        }
public:
        FASTFIFO(UCHAR* pMemory, size_t Size) :
                pBegin(pMemory),
                pEnd(pMemory + Size) {

                Clear();
        }

        VOID    Put(char c) {           // NO OVERRUN CHECKING
                *pWrite = (UCHAR) c;
		pWrite = Next(pWrite);
                return;
        }

        INT     Get() {
                if (pRead == pWrite) {
                        return FASTFIFO_EOF;
                }
                INT nReturnVal = *pRead;
                pRead = Next(pRead);
                return nReturnVal;
        }
        
        VOID    Clear() {
                pRead = pWrite = pBegin;
        }

};
#endif // fastfifo.hpp
