/****************************************************************************
Date: 3/18/2002
File: ft736.hpp
Description:
	Yaseu FT-736 radio driver

History:
	12/1/97 Test version 1
	1/9/98  Added keyboard tuning 
	3/18/2002 Add debug stuff
*****************************************************************************/
#ifndef FT736_HPP
#define FT736_HPP

#include "loctypes.h"
#include "radio.hpp"	// RADIO base class
#include "comdrv.hpp"	// COM port driver class
#include "inttimer.hpp"	// interval timer
#include "kbdtune.hpp"	// keyboard tuner

//***** Frequency Steps ****************
const LONG FT736_DEFAULTSTEP = 10;	// 10 Hz "normal" step size
const LONG FT736_FMSTEP = 100;		// 100 Hz FM step size

//***** Message Definitions ************
// Message consists of 4 octets for
// parameters and 1 octet for opcode.
// Opcode is always last.
const INT  FT736_MESSAGEPARAMS	= 4;  	// number of "bytes" in params
const INT  FT736_MESSAGESIZE	= 5;  	// number of "bytes" in message

//****** OpCodes ***********************
// These are used to build up commands
const UCHAR FT736_CATON		= 0x00;	// Computer-Aided Transceiver (ON)
const UCHAR FT736_CATOFF	= 0x80;	// Computer-Aided Transceiver (OFF)
const UCHAR FT736_SETFREQ	= 0x01;	// Set simplex VFO
const UCHAR FT736_SETMODE	= 0x07; // Set simplex mode
const UCHAR FT736_FULLDUPLEXON	= 0x0e;	// Full duplex (sat mode) ON
const UCHAR FT736_FULLDUPLEXOFF	= 0x0e;	// Full duplex (sat mode) OFF
const UCHAR FT736_DUPLEXRXMODE	= 0x17;	// Set full duplex Rx modulation
const UCHAR FT736_DUPLEXTXMODE	= 0x27;	// Set full duplex Tx modulation
const UCHAR FT736_DUPLEXRXFREQ	= 0x1e;	// Set full duplex Rx frequency
const UCHAR FT736_DUPLEXTXFREQ	= 0x2e;	// Set full duplex Tx frequency
const UCHAR FT736_GETSMETER	= 0xf7;	// Get S-meter value
const UCHAR FT736_TEST		= 0xf7; // use get S-meter msg to test radio

//***** COM PORT PARAMETERS ******************************************
const USHORT	FT736_TIMEOUT	= 1;	// 1 second response timer
const UINT	FT736_BITRATE	= 4800;
const UINT	FT736_DATABITS	= 8;
const UINT	FT736_STOPBITS	= 2;
const UARTPARITY FT736_PARITY	= UARTPARITY_NONE;

class FT736 : public RADIO {
private:
	// Radio Control OPCODES
	UCHAR		ucSetRxFreq;	// OPCODE to use to set rx freq
	UCHAR		ucSetRxMode;	// OPCODE to use to set rx mode
	UCHAR		ucSetTxFreq;	// OPCODE to use to set tx freq
	UCHAR		ucSetTxMode;	// OPCODE to use to set tx mode

	// Rx Params
	LONG		lRxStepSize;	// Frequency Step Size
	LONG		lRxVFOSteps;	// receiver VFO steps
	LONG		lRxRITSteps;	// receiver independent tuning steps
	LONG		lRxActualSteps;	// current receiver actual tuning steps
	MODULATION	eRxMode;	// current rx modulation type

	// Tx Params
	LONG		lTxStepSize;	// Frequency Step Size
	LONG		lTxVFOSteps;	// transmitter VFO steps
	LONG		lTxActualSteps;	// current Tx actual frequency steps
	MODULATION	eTxMode;	// current Tx modulation type
	
	// Com port driver
	COMDRV		Comport;
	
	// Structure of messages sent to the radio
	struct {
		UCHAR	aParams[FT736_MESSAGEPARAMS];
		UCHAR	ucOpcode;
	} Message;

	// Keyboard Tuner Driver
	KEYBOARDTUNER	KbdTuner;

	//******************************************************
	// Utilities
	//******************************************************

	// Send a message to the radio
	VOID	SendMessage(UCHAR ucOpcode) {
		Message.ucOpcode = ucOpcode;
		Comport.Write((const VOID*) &Message, FT736_MESSAGESIZE);
	}

	// Table to convert MODULATION type to FT-736 message param
	static const UCHAR aSetModeTable[];

	// Set actual radio mode
	VOID	SetMode(UCHAR ucOpcode, MODULATION eMode) {
		Message.aParams[0] = aSetModeTable[eMode];
		SendMessage(ucOpcode);
	}

	// Convert Frequency to steps
	LONG	FreqtoSteps(DOUBLE dFrequency, LONG lStepSize) {

		//************************************************
		// WARNING: this code has a workaround for MS C7 bug
		// converting double to long. Do NOT change!
		//************************************************
		volatile ULONG ulFrequency = (ULONG) dFrequency;
		LONG lFrequency = (LONG) ulFrequency;
		return ( (lFrequency) + (lStepSize/2) ) / lStepSize;
	}


	// Set actual radio frequency
	VOID SetFrequency(LONG	lFrequencySteps,
			  LONG	lStepSize,
			  UCHAR	ucOpcode);

	LONG GetStepSize(MODULATION eMode) {
	  return (eMode==MODULATION_FM) ? FT736_FMSTEP : FT736_DEFAULTSTEP;
	}


	// No copy constructor or assignment allowed
	// Force compile error if attempted
	FT736(const FT736&);
	FT736& operator=(const FT736&);

public:
	// General Functions
	FT736(const RADIOPARAMS& Params);
	~FT736();
	VOID	Update();
	VOID	Debug();

	// Tx Functions
	VOID	SetTxFreq(DOUBLE dFrequency);
	VOID	SetTxMode(MODULATION eMode);

	// Rx Functions
	BOOL	HasUserTunedRx();
	VOID	RefreshRxParams();
	DOUBLE	GetRxFreq();
	MODULATION GetRxMode();
	VOID	SetRxFreq(DOUBLE dFrequency);
	VOID	SetRxMode(MODULATION eMode);
};
#endif // end ft736.hpp
