/****************************************************************************
Date: 8/30/00
File: ft817.cpp
Description:
	FT817 radio driver G6LVB

History:
	A Version - based on FT100 K version

*****************************************************************************/
#include "ft817.hpp"


// Set Mode Table
// Index into table is MODULATION type, returns FT-817 mode code
const UCHAR FT817::aSetModeTable[] = {
	0x01,	// NA	(USB is default)
	0x00,	// LSB	
	0x01,	// USB
	0x02,	// CW
	0x08,	// FM
	0x04,	// AM
	0x0C	// FSK (PKT)
};

// Get Mode Table
// index into table is lower 4 bits of FT-817 code, returns MODULATION type
const MODULATION FT817::aGetModeTable[] = {
		MODULATION_LSB,	// 0
		MODULATION_USB,	// 1
		MODULATION_CW,	// 2 
		MODULATION_CW,	// 3 (CW-R)
		MODULATION_AM,	// 4
		MODULATION_NA,	// 5
		MODULATION_FM,	// 6 FM Wide
		MODULATION_NA,	// 7
		MODULATION_FM,	// 8
		MODULATION_NA,	// 9
		MODULATION_FSK,	// 10
		MODULATION_NA,	// 11
		MODULATION_FSK,	// 12 PKT
		MODULATION_NA,	// 13
		MODULATION_NA,	// 14
		MODULATION_NA,	// 15
};


/****************************************************************************
Function: SetFrequency()				PRIVATE
Description: Set radio to this frequency
Parameters:
	Steps = desired frequency in steps
Returns:
*****************************************************************************/
VOID	FT817::SetFrequency(LONG lFrequencySteps, LONG lStepSize) {

	// Convert Steps to frequency
	LONG	lFrequency = lFrequencySteps * lStepSize;

	//********************************************
	// Convert frequency to "Yaesu BCD" and
	// save in message params buffer
	// Note that frequency bytes are sent LSB first,
	// (Starting at Param1.)
	//********************************************
	LONG lDivisor	  = 100000000L; 	// (i.e. 100 MHz)
	LONG lTemp;
    for (SHORT nParam = FT817_MESSAGEP4; nParam >=0;  nParam--) {

		// Make upper nibble
		lTemp = lFrequency / lDivisor;
		lFrequency -= (lTemp * lDivisor);
		OutputBuffer.aParams[3-nParam] = (UCHAR) lTemp << 4;
		lDivisor /= 10;

		// Add in lower nibble
		lTemp = lFrequency / lDivisor;
		lFrequency -= (lTemp * lDivisor);
		OutputBuffer.aParams[3-nParam] += (UCHAR) lTemp;
		lDivisor /= 10;
	}

	SendMessage(FT817_SETFREQ);
	return;
}

/****************************************************************************
Function: FT817()
Description: constructor
Parameters:
	RADIOPARAMS structure
Returns:
*****************************************************************************/
FT817::FT817(const RADIOPARAMS& Params) :
	
	// Initialize
	RADIO(),
	Timer(),
	eState(STATE_IDLE),
	unMessageSize(0),
	ePTT(BOOL_FALSE),
	lRxVFOSteps(0),
	lRxVFORead(0),
	ucRxBand(0xff),
	eRxMode(MODULATION_NA),
	eUserHasTunedRx(BOOL_FALSE),
	lTxVFOSteps(0),
	eTxMode(MODULATION_NA),
	eTxLockout(BOOL_FALSE),
	Comport(),
	unByteCounter(0) {

	//********************************************************
	// Open com port and check communications with radio
	// If fails, return with Operational flag FALSE
	//********************************************************
	RADIO::eOperational = Comport.Open(
					Params.unPort,
					Params.unRate ?
						Params.unRate :
						FT817_BITRATE,
					FT817_DATABITS,
					FT817_PARITY,
					FT817_STOPBITS,
					Params.unIRQ,
					Params.Address);

	if (!RADIO::eOperational) {
		return;		// failed to open
	}

	//**********************************************************
	// Check communications with Radio.
	// Send test message to Radio and start a 1 second timer.
	// If timer expires before we receive test message response,
	// communications has failed.
	//***********************************************************

	// Some Laptop COM ports need a wake-up and radio seems to ignore
	// first message so send a harmless one and wait
	SendMessage(FT817_SPLITMODE_OFF);

    while(!Timer.IsExpired() || Comport.Getc() != COMDRV_EOF) { }

	// Now see if radio is talking
	SendMessage(FT817_READFREQMODE);
	USHORT		unCounter = 0;
	while(unCounter < FT817_READFREQMODESIZE) { 
		if (Timer.IsExpired()) {
 			// Radio failed to communicate
			RADIO::eOperational = BOOL_FALSE;
			return;
		}
		if (Comport.Getc() != COMDRV_EOF) {
			unCounter++;
		} 
	} // Radio is talking!

	if (Params.eTransceive)
	{
		// Always run in Split mode
		// A is receive, VFO B is TX
		// Select VFO A for Rx
		SendMessage(FT817_SPLITMODE_ON);
		eState=STATE_WAITACK;
		unMessageSize=FT817_SPLITMODE_ONSIZE;
		eSplitMode = BOOL_TRUE;
	}
	else
	{
		SendMessage(FT817_SPLITMODE_OFF);
		eState=STATE_WAITACK;
		unMessageSize=FT817_SPLITMODE_OFFSIZE;
		eSplitMode = BOOL_FALSE;
	}

	// Always run in Split mode
	return;
}

/****************************************************************************
Function: 	~FT817()
Description:	destructor
Parameters:
Returns:
*****************************************************************************/
FT817::~FT817() {

	SendMessage(FT817_SPLITMODE_OFF);
    while(!Timer.IsExpired()) { }   // wait 1 sec
	Comport.Close();
	return;
}

/****************************************************************************
Function: 	IsReady()
Description:
		Check if radio is ready for next command
Parameters:
Returns:
		BOOL_TRUE = Ready
		BOOL_FALSE = Not Ready
*****************************************************************************/

BOOL FT817::IsReady() { 
	if (eState == STATE_IDLE) {
		return BOOL_TRUE;
	}
	return BOOL_FALSE;
}

/****************************************************************************
Function: Update()
Description:
		Processes messages received from radio
Parameters:
	
Returns:
*****************************************************************************/
VOID	FT817::Update() {

	if (eState == STATE_IDLE) {
		return;			// do nothing
	}

	// Check for next message byte received
	INT nByte = Comport.Getc();
	if (nByte == COMDRV_EOF) {

		// Nope, Check for Timeout
		if (Timer.IsExpired()) {
			unByteCounter = 0;
			Comport.ClearBuffers();
			RefreshRxParams();
		}
		return;
	}

	// Got a byte, save it
	aInputBuffer[unByteCounter++] = (UCHAR) nByte;

	// if last byte in message, process it
	if ( unByteCounter == unMessageSize) {

		Timer.Stop();		// stop message timeout timer
		unByteCounter = 0;	// reset 

		switch(eState) {
		case STATE_READTXSTATUS:
			// Get PTT status
			if (aInputBuffer[0]!=FT817_NOPTT) {
				ePTT = BOOL_TRUE;
				eTxLockout = BOOL_TRUE;
				eState = STATE_IDLE;
			}
			else {
				ePTT = BOOL_FALSE;
				// Not transmitting, send status update
				SendMessage(FT817_READFREQMODE);
				eState = STATE_READFREQMODE;
				unMessageSize = FT817_READFREQMODESIZE;
			}
			break;

		case STATE_READFREQMODE:

			// Check Rx band so we don't read in the
			// Tx band (workaround for radio CAT problem)

			// Is this the first update?
			if (ucRxBand == 0xff) {
				// Yes, set Rx band
				ucRxBand = aInputBuffer[0];
			}

			// Check read Rx band against saved Rx band
			if (ucRxBand != aInputBuffer[0]) {
				// No, OOPS! this is the Tx band

				// force freqs and modes to be
				// re-sent to radio
				lRxVFOSteps = 0;
				eRxMode = MODULATION_NA;

				lTxVFOSteps = 0;
				eTxMode = MODULATION_NA;

				// ignore this poll
				eState = STATE_IDLE;
			}
			else { 	// band is OK

				// Get Rx Mode
				ucRxModeRead = aInputBuffer[4];

				// Get Rx VFO Freq
				lRxVFORead = 0;
				for (INT i = 0; i <= 3; i++) {
					// Save Rx VFO
                    lRxVFORead = lRxVFORead*100 +
                        ((aInputBuffer[i] >> 4) & 0x0F) * 10 +
                        (aInputBuffer[i] & 0x0F);
				}

                // Send "Read TX Status" again to check PTT
				SendMessage(FT817_READTXSTATUS);
				eState = STATE_CHECK;
				unMessageSize = FT817_READTXSTATUSSIZE;
			}
			break;

		case STATE_CHECK:
			// Get PTT status
			if (aInputBuffer[0]!=FT817_NOPTT) {
				ePTT = BOOL_TRUE;
				eTxLockout = BOOL_TRUE;
			}
			else {
				// Not transmitting, save Rx params
				ePTT = BOOL_FALSE;

				// Convert mode code to MODULATION type and save
				eRxMode = aGetModeTable[ucRxModeRead & 0x0F];
				// Make into steps
                lRxVFORead = lRxVFORead * 10L / FT817_RXSTEPSIZE;

				// See if user has manually tuned radio
				// if so, save new freq steps
				if (lRxVFORead != lRxVFOSteps) {
					eUserHasTunedRx = BOOL_TRUE;
					lRxVFOSteps = lRxVFORead;

					eTxLockout = BOOL_TRUE;
				}
				else {
					// user has not tuned
					eTxLockout = BOOL_FALSE;
				}
			}
			eState = STATE_IDLE;
			break;

        case STATE_WAITACK:
            // Discard ACK/NACK from assorted FT-817 commands
            eState = STATE_IDLE;
            break;

        case STATE_SETTXFREQ1:
            SetFrequency(lTxVFOSteps, FT817_TXSTEPSIZE); // freq set
            eState = STATE_SETTXFREQ2;
            unMessageSize = FT817_SETFREQSIZE;
            break;

        case STATE_SETTXFREQ2:
            SendMessage(FT817_VFOMODE);
            eState = STATE_WAITACK;
            unMessageSize = FT817_VFOMODESIZE;
            break;

        case STATE_SETTXMODE1:
            SetMode(eTxMode); // mode set
            eState = STATE_SETTXMODE2;
            unMessageSize = FT817_SETMODESIZE;
            break;

        case STATE_SETTXMODE2:
            SendMessage(FT817_VFOMODE);
            eState = STATE_WAITACK;
            unMessageSize = FT817_VFOMODESIZE;
            break;

		default:
			// error case
			break;
		} // end switch()

	} // end of process received message

	return;
}

/****************************************************************************
Function: SetTxFreq()
Description:
	Set transmit frequency if different from current frequency
Parameters:
	Frequency
Returns:
*****************************************************************************/
VOID	FT817::SetTxFreq(DOUBLE dFrequency) {

	// If TX lockout is TRUE, ignore
	if (eTxLockout) {
		return;
	}

	// Calculate new frequency steps 
	LONG lNewTxVFOSteps = FreqtoSteps(dFrequency, FT817_TXSTEPSIZE);

	// Check to see if freq steps has changed
	// if so, update VFO steps and send new freq to radio
	if (lTxVFOSteps != lNewTxVFOSteps) {
		lTxVFOSteps = lNewTxVFOSteps;

		if (eSplitMode) // G6LVB 13 APR 2001 only switch VFO's in XCVR mode
		{
			SendMessage(FT817_VFOMODE);
			eState = STATE_SETTXFREQ1;
			unMessageSize = FT817_SETMODESIZE;
		}
		else
		{ // G6LVB 13 APR 2001 No need to switch VFO back when not in XCVR mode
			SetFrequency(lTxVFOSteps, FT817_TXSTEPSIZE); // freq set
			eState = STATE_WAITACK;
			unMessageSize = FT817_SETFREQSIZE;
		}
	}
	return;
}

/****************************************************************************
Function: SetTxMode()
Description:
	Set transmitter modulation mode type
Parameters:
	Mode
Returns:
*****************************************************************************/
VOID	FT817::SetTxMode(MODULATION eMode) {

	if (eTxLockout) {
		return;
	}

	// Send message if different from current tx mode
	if (eTxMode != eMode) {
		eTxMode = eMode;	// Save new tx mode

		if (eSplitMode) // G6LVB 13 APR 2001 only switch VFO's in XCVR mode
		{
	        SendMessage(FT817_VFOMODE);
		    eState = STATE_SETTXMODE1;
			unMessageSize = FT817_VFOMODESIZE;
		}
		else
		{ // G6LVB 13 APR 2001 No need to switch VFO back when not in XCVR mode
            SetMode(eTxMode); // mode set
			eState = STATE_WAITACK;
            unMessageSize = FT817_SETMODESIZE;
		}
	}
	return;
}

/****************************************************************************
Function: RefreshRxParams()
Description:
	Starts polling cycle
Parameters:
Returns:
*****************************************************************************/
VOID	FT817::RefreshRxParams() {

	SendMessage(FT817_READTXSTATUS);
	unMessageSize = FT817_READTXSTATUSSIZE;
	eState = STATE_READTXSTATUS;
	return;
}

/****************************************************************************
Function: GetRxFreq()
Description:
	Gets Receive (VFO) frequency
Parameters:
Returns:
	Frequency
*****************************************************************************/
DOUBLE	FT817::GetRxFreq() {

	eUserHasTunedRx = BOOL_FALSE;
	return (DOUBLE) (lRxVFOSteps * FT817_RXSTEPSIZE);
}

/****************************************************************************
Function: SetRxFreq()
Description:
	Set Receive frequency
Parameters:
	Frequency
Returns:
*****************************************************************************/
VOID	FT817::SetRxFreq(DOUBLE dFrequency) {

	eUserHasTunedRx = BOOL_FALSE;

	if (ePTT) {		// can't do it if transmitting
		return;
	}

	LONG lNewRxVFOSteps = FreqtoSteps(dFrequency, FT817_RXSTEPSIZE);
	if (lRxVFOSteps != lNewRxVFOSteps) {
		lRxVFOSteps = lNewRxVFOSteps;

		// G6LVB - not sure exactly how to stop user PTT'ing here...

		SetFrequency(lNewRxVFOSteps, FT817_RXSTEPSIZE);
        eState=STATE_WAITACK;
        unMessageSize=FT817_SETFREQSIZE;
	}

	return;
}

/****************************************************************************
Function: SetRxMode()
Description:
	Set receive modulation mode type.
Parameters:
	Mode
Returns:
*****************************************************************************/
VOID	FT817::SetRxMode(MODULATION eMode) {

	if (ePTT) {		// cannot do it if transmitting
		return;
	}

	// Change if new mode is different from current mode
	if (eRxMode != eMode) {
		SetMode(eMode);		// Send message to radio
		eRxMode = eMode; 	// Save new Rx mode
	}
	return;
}

// end FT817.cpp
