/****************************************************************************
Date: 3/9/2002
File: icomhf.cpp
Description:
	ICOM HF Radio Driver
History:
	1/15/2002 initial version
	2/26/2002 R-8500
	3/6/2002 debug for R-8500
	3/9/2002 Fix tuning step problem
*****************************************************************************/
#include "icomhf.hpp"

/****************************************************************************
Function: Constructor
Description:
Parameters:
Returns:
*****************************************************************************/
ICOMHF::ICOMHF(const RADIOPARAMS& Params) :

	RADIO(),			// Base class
	Msg(),				// Msg Handler
	eReady(BOOL_FALSE),
	lRxVFOSteps(0),
	eRxMode(MODULATION_NA),
	eUserHasTunedRx(BOOL_FALSE),
	lTxVFOSteps(0),
	eTxMode(MODULATION_NA) {

	RADIO::eOperational = Msg.Open( Params.unParam1,	// CI-V Address
					Params.unPort,		// COM port
					Params.unRate,		// Bit rate
					Params.unIRQ,		// HW - IRQ
					Params.Address);	// HW IO address

	if (!RADIO::eOperational) {
		return;		// Msg handler failed to open COM port
	}

	// Radio Ready
	eReady = BOOL_TRUE;
	return;
}

/****************************************************************************
Function: Update() 
Description:
	This is the state machine that processes all
	radio messages. It runs every clock tick.
Parameters:
Returns:
*****************************************************************************/
VOID	ICOMHF::Update() {

	// If READY flag is set, nothing to do
	if (eReady) {
		return;
	}

	// Check for a received message
	if (!Msg.IsReady()) {
		return;			// No message, nothing to do
	}

	// Process Message based on command type
	switch(Msg.GetCmd()) {

		LONG lNewRxVFOSteps;	// temp

	case ICOMMSG_READFREQ:
		lNewRxVFOSteps = GetSteps(Msg.GetFrequency(), eRxMode);
		if (lNewRxVFOSteps != lRxVFOSteps) {
			eUserHasTunedRx = BOOL_TRUE;
			lRxVFOSteps = lNewRxVFOSteps;
		}
		eReady = BOOL_TRUE;
	    	break;	

	case ICOMMSG_READMODE:
		eRxMode = Msg.GetMode();
		// Got the RX mode, now get the RX freq
		Msg.SendCmd(ICOMMSG_READFREQ);
		break;

	case ICOMMSG_OK:
		eReady = BOOL_TRUE;
		break;
		break; // end of ICOMMSG_OK processing

	default: // ignore
		eReady = BOOL_TRUE;
		break;
	} // end msg processing

	return;
}

/****************************************************************************
Function: Destructor
Description:
Parameters:
Returns:
*****************************************************************************/
ICOMHF::~ICOMHF() {
	Msg.Close();
}


/****************************************************************************
Function: SetTxFreq()
Description:
	Sets Transmitter frequency
Parameters:
	DOUBLE 	dFrequency
Returns:
*****************************************************************************/
VOID	ICOMHF::SetTxFreq(DOUBLE dFrequency) {

	//***** Get new TX VFO steps *****
	LONG lFrequency = FreqtoLong(dFrequency);
	LONG lNewTxVFOSteps = GetSteps(lFrequency, eTxMode);
	if (lNewTxVFOSteps == lTxVFOSteps) {
		return;				// no change
	}
	lTxVFOSteps = lNewTxVFOSteps;		// Save New Tx Freq Steps

	Msg.SendWriteFrequency(lTxVFOSteps * GetStepSize(eTxMode));	

	eReady = BOOL_FALSE;
	return;
}


/****************************************************************************
Function: SetTxMode()
Description: Set transmit modulation type
Parameters:
	MODULATION eMode
Returns:
*****************************************************************************/
VOID	ICOMHF::SetTxMode(MODULATION eMode) {

	if (eMode == eTxMode) { 		// if Mode has changed...
		return;
	}
	eTxMode = eMode; 		// Save New Tx Mode

	Msg.SendWriteMode(eMode);	

	eReady = BOOL_FALSE;
	return;
}

/****************************************************************************
Function: RefreshRxParams()
Description:
	Poll radio to get receiver frequency and mode
Parameters:
Returns:
*****************************************************************************/
VOID	ICOMHF::RefreshRxParams() {

	Msg.SendCmd(ICOMMSG_READMODE); 

	eReady = BOOL_FALSE;
	return;
}

/****************************************************************************
Function: GetRxFreq()
Description:
	Returns current receiver frequency from previously
	refreshed rx params
Parameters:
Returns:
	DOUBLE	dFrequency
*****************************************************************************/
DOUBLE	ICOMHF::GetRxFreq() {

	// Set flag to show that user has not tuned from this returned freq
	eUserHasTunedRx = BOOL_FALSE;

	// Convert RX VFO Frequency Steps to Double
	LONG lFrequency = lRxVFOSteps * GetStepSize(eRxMode);
	return (DOUBLE) lFrequency;
}


/****************************************************************************
Function: SetRxFreq()
Description:
	Sets Receiver frequency
Parameters:
	DOUBLE 	dFrequency
Returns:
*****************************************************************************/
VOID	ICOMHF::SetRxFreq(DOUBLE dFrequency) {

	//***** clear user has tuned flag ****
	eUserHasTunedRx = BOOL_FALSE;

	//***** Get new RX VFO steps *****
	LONG lFrequency = FreqtoLong(dFrequency);
	LONG lNewRxVFOSteps = GetSteps(lFrequency, eRxMode);
	if (lNewRxVFOSteps == lRxVFOSteps) {
		return;				// unchanged
	}
	lRxVFOSteps = lNewRxVFOSteps; 	// Save New Rx Frequency Steps
	
	Msg.SendWriteFrequency(lRxVFOSteps * GetStepSize(eRxMode));

	eReady = BOOL_FALSE;
	return;
}

/****************************************************************************
Function: SetRxMode()
Description:
	Sets receiver modulation
Parameters:
	MODULATION eMode
Returns:
*****************************************************************************/
VOID	ICOMHF::SetRxMode(MODULATION eMode) {

	if (eMode == eRxMode) {
		return;				// unchanged
	}
	eRxMode = eMode;			// save new mode


	Msg.SendWriteMode(eMode);	

	eReady = BOOL_FALSE;
	return;
}
// end icomhf.cpp
