/****************************************************************************
Date: 1/20/98
File: kbdtune.cpp

Description:	Keyboard Tuner Class
	
Public:
	KEYBOARDTUNER()
	~KEYBOARDTUNER()

	// NOTE: Interrupts must be disabled before using these
	VOID	Activate()
	VOID	Deactivate()
	BOOL	HasUserTuned()
	LONG	GetVFOSteps()
	LONG	GetRITSteps()
	MODULATION GetModulation()

History:
	12/26/97 Initial Version
	1/20/98	 Added RIT step size via SHIFT keys and x1K steps

*****************************************************************************/
#include "kbdtune.hpp"

// DOS Keyboard constants
const INT KBDTUNE_IRQ = 1;	// Keyboard uses IRQ1
const INT KBDTUNE_DATAREG = 0x60;
const INT KBDTUNE_CONTROLREG = 0x61;

// bios shift status byte
const UCHAR FAR* pBiosShiftStatus = (const UCHAR FAR*)  MK_FP(0x40,0x17);
const UCHAR KBDTUNE_CAPSLOCK = 0x40; // bit in bios shift status byte

// tuning scan codes
const KBDTUNE_UPVFO	= 0x4d;	// right Arrow	
const KBDTUNE_DOWNVFO	= 0x4b;	// left Arrow	
const KBDTUNE_UPRIT	= 0x48;	// Up Arrow	
const KBDTUNE_DOWNRIT	= 0x50;	// Down Arrow	
const KBDTUNE_AM	= 0x1e;	// A
const KBDTUNE_FM	= 0x21; // F
const KBDTUNE_CW	= 0x2e;	// C
const KBDTUNE_LSB	= 0x26; // L
const KBDTUNE_USB	= 0x16; // U

// Table with step multipliers
// use lower 3 bits of shift status as index
// bit0 = RIGHTSHIFT, bit1=LEFTSHIFT, bit2=CTRL
const USHORT aStepTable[] = {
	1,	// no shift keys
	100,	// right shift 
	100,	// left shift
	100,	// left and right shift
	10,	// CTRL
	1000,	// CTRL + right
	1000,	// CTRL + left
	1000	// CTRL + both
};

// local variables;
static PICDRV		PicDrv;			// PIC driver
static BOOL 		eInUse = BOOL_FALSE;
static BOOL		eUserHasTuned;
static volatile LONG	lVFOSteps;
static volatile LONG	lRITSteps;
static volatile MODULATION eMode;

// Interrupt Service Routine
VOID	INTERRUPT KbdTunerISR() {

	// Get keyboard shift status 
	UCHAR	ucShiftStatus = *pBiosShiftStatus;
	
	// if CAPSLOCK is not on, chain to old ISR
	if (!(ucShiftStatus & KBDTUNE_CAPSLOCK)) {
		PicDrv.Chain();
	}

	// Get step adjustment
	LONG lStepAdjust = aStepTable[(ucShiftStatus & 0x07)];

	// Process SCAN Code
	switch ( inp(KBDTUNE_DATAREG) ) {
	case KBDTUNE_UPVFO:
		lVFOSteps += lStepAdjust;
		eUserHasTuned = BOOL_TRUE;
		break;
	case KBDTUNE_DOWNVFO:
		lVFOSteps -= lStepAdjust;
		eUserHasTuned = BOOL_TRUE;
		break;
	case KBDTUNE_UPRIT:
		lRITSteps += lStepAdjust;
		break;
	case KBDTUNE_DOWNRIT:
		lRITSteps -= lStepAdjust;
		break;
	case KBDTUNE_AM:
		eMode = MODULATION_AM;
		break;
	case KBDTUNE_FM:
		eMode = MODULATION_FM;
		break;
	case KBDTUNE_CW:
		eMode = MODULATION_CW;
		break;
	case KBDTUNE_LSB:
		eMode = MODULATION_LSB;
		break;
	case KBDTUNE_USB:
		eMode = MODULATION_USB;
		break;
	default:	// not a tuning key, chain to old ISR
		PicDrv.Chain();
	}	

	// Reset Keyboard Controller
	INT nTemp = inp(KBDTUNE_CONTROLREG);
	outp(KBDTUNE_CONTROLREG, nTemp | 0x80); // set MSB
	outp(KBDTUNE_CONTROLREG, nTemp);	// clear MSB

	// Reset PIC
	PicDrv.EndOfInterrupt();

	return;	
}

// constructor
KEYBOARDTUNER::KEYBOARDTUNER() : eActive(BOOL_FALSE) {}

// destructor
KEYBOARDTUNER::~KEYBOARDTUNER() { Deactivate(); }

VOID KEYBOARDTUNER::Activate() {
	if (!eInUse) {
		PicDrv.Install(KBDTUNE_IRQ, (INTHANDLER) KbdTunerISR);
		eInUse = BOOL_TRUE;
		eActive = BOOL_TRUE;
		lVFOSteps = 0;
		lRITSteps = 0;
		eUserHasTuned = BOOL_FALSE;
		eMode = MODULATION_NA;
	}
	return;
}

VOID KEYBOARDTUNER::Deactivate() {
	if (eActive) {
		eActive = BOOL_FALSE;
		eInUse = BOOL_FALSE;
		PicDrv.Restore();
	}
	return;
}

BOOL	KEYBOARDTUNER::HasUserTuned() {
	if (!eActive) {
		return BOOL_FALSE;
	}
	BOOL eTemp = eUserHasTuned;
	eUserHasTuned = BOOL_FALSE;
	return eTemp;
}
	
LONG	KEYBOARDTUNER::GetVFOSteps() {
	if (!eActive) {
		return 0;
	}
	LONG lSteps = lVFOSteps;
	lVFOSteps = 0;
	return lSteps;
}

LONG	KEYBOARDTUNER::GetRITSteps() {
	if (!eActive) {
		return 0;
	}
	LONG lSteps = lRITSteps;
	lRITSteps = 0;
	return lSteps;
}

MODULATION KEYBOARDTUNER::GetModulation() {
	if (!eActive) {
		return MODULATION_NA;
	}
	MODULATION eModulation = eMode;
	eMode = MODULATION_NA;
	return eModulation;
}
	
// end kbdtune.cpp
