/****************************************************************************
Date: 3/3/97 EXPERIMENTAL
File: parser.hpp
Description:	LINE-ORIENTED, TEXT-FILE PARSER CLASS

	This class provides a basic line-oriented parser for ascii
	text files. The basic structure of a line in the file is:

		TOKEN	{ STRING }

	Where TOKEN is some keyword followed by any number of strings.

	This parser also handles comments. If a comment character is
	found at the beginning of any string, the rest of that line is
	discarded. 

	GetToken() gets the next line from the file and returns the first
	string on the line or NULL if at the end of the file.

	GetString() returns the next string on the same line or "" if there
	are no more.

	GetLine() returns a pointer to an unmodified copy of the original
	text line as read from the file. This may be useful for error reporting.

	GetLineNumber() returns the current line number read from the 
	text file or 0 if no lines have been read. 

	GetFloat() converts the next string on the same line to a 
	floating point number. If there are no strings left, it returns 0.

	GetInteger() converts the next string on the same line to an integer.
	If there are no strings left, it returns 0.

	GetHex() interprets the next string on the same line as a hexadecimal
	number. If there are no strings left, it returns 0.

public:
	PARSER(CHAR cComment, const CHAR* pszSeperators)
	~PARSER()
	BOOL	Open(const CHAR* pszFileName)
	BOOL	IsEOF() const
	VOID	Close()
	UINT	GetLineNumber() const 
	CHAR*	GetLine()
	CHAR*	GetToken()
	CHAR*	GetString()
	DOUBLE	GetFloat()
	LONG 	GetInteger()
	ULONG	GetHex()

History:
*****************************************************************************/
#ifndef PARSER_HPP
#define PARSER_HPP

#include <stdlib.h>
#include <string.h>
#include "loctypes.h"

// Max number of characters per line, any beyond this are ignored
const UINT PARSER_MAXLINE = 80;

// Default comment character and token/string seperators
const CHAR  PARSER_COMMENT = ';' ;	
const CHAR  PARSER_ENDL = '\n';
#define PARSER_WHITESPACE  "\n \t\r\v"

class PARSER {
private:
	const CHAR COMMENT;		// comment character
	const CHAR* const WHITESPACE;	// token seperators

	// File status
	BOOL	eOpen;
	BOOL	eEOF;			// end of file
	INT	nFileHandle;
	UINT	unLine;			// current file line number

	// Line Buffers
	// szBuffer is the working buffer which is modified 
	// szLine is the unmodified line as read from the file
	enum { BUFFSIZE = PARSER_MAXLINE + 1 };
	CHAR	szBuffer[BUFFSIZE];
	CHAR	szLine[BUFFSIZE];

	// ReadLine utility, reads a text line into buffer
	UINT	ReadLine(CHAR FAR* aBuffer, UINT unBuffSize, INT nFileHandle);

	// no copy or = operators
	PARSER& operator =(const PARSER&);
	PARSER(const PARSER&);

public:
	PARSER();
	PARSER(CHAR cComment, const CHAR* pszWhiteSpace);
	~PARSER();

	BOOL	Open(const CHAR* pszFileName);
	BOOL	IsEOF() { return eEOF; }
	VOID	Close();

	UINT	GetLineNumber() const { return unLine; }
	CHAR*	GetLine() { return szLine; }
	CHAR*	GetToken();
	CHAR*	GetString() {
		CHAR* pszString = strtok(NULL, WHITESPACE);
		return pszString ? pszString : "";
	}
	DOUBLE	GetFloat() { return strtod(GetString(), NULL); }
	LONG	GetInteger() {
			return  strtol(GetString(), NULL, 10);
	}
	ULONG	GetHex() {
			return  strtoul(GetString(), NULL, 16);
	}
};
#endif
