/****************************************************************************
Date: 13 APR 2001
File: ts2000.hpp
Description:
	ts2000 radio driver

History:
	13 APR 2001 G6LVB

*****************************************************************************/
#ifndef TS2000_HPP
#define TS2000_HPP

#include "loctypes.h"
#include "radio.hpp"	// RADIO base class
#include "comdrv.hpp"	// COM port driver class
#include "inttimer.hpp"	// interval timer


//***** Frequency Steps ****************
const LONG TS2000_STEPSIZE	= 10;	// 10 Hz frequency tuning step size
const LONG TS2000_1GIGAHERTZ	= 1000000000L; // 1GHz

//***** Message Definitions ************
const INT TS2000_MAXMESSAGE	= 80;  	// number of bytes in message in or out
const INT TS2000_OPCODESIZE = 2;	// all opcodes are 2 chars
const CHAR* const TS2000_ENDOFMESSAGE = ";"; // End of message delimiter

//****** OpCodes ***********************
// Radio command codes
const CHAR* const TS2000_MODE	= "MD"; // set mode (+ 1 digit)
const CHAR* const TS2000_VFOA	= "FA"; // set or read VFO A
const CHAR* const TS2000_VFOB	= "FB"; // set or read VFO B
const CHAR* const TS2000_STATUS	= "FA;MD"; // retrieve transceiver status
const CHAR* const TS2000_TEST	= " ";	// Test message
const CHAR* const TS2000_SATMODEON	= "SA1010000;XT0;RT0;FA;FB"; // Satellite mode on
const CHAR* const TS2000_SATMODEOFF	= "SA0000000;FR0;FT0;XT0;RT0"; // Satellite mode off
//const CHAR* const TS2000_SATMODEON	= "SA1010000;FR0;FT0;XT0;RT0"; // Satellite mode on
//const CHAR* const TS2000_SATMODEOFF	= "SA0000000;FR0;FT0;XT0;RT0"; // Satellite mode off
const CHAR* const TS2000_SATONTXMODEPRE		= "LK21;SA1011000;MD";
const CHAR* const TS2000_SATONTXMODEPOST	= ";SA1010000;LK00";
const CHAR* const TS2000_SATOFFTXMODEPRE	= "MD";
const CHAR* const TS2000_SATOFFTXMODEPOST	= "";

const UINT TS2000_TESTSIZE	= 2; // Expected return message size
const UINT TS2000_SATMODEONSIZE = 28; // Expected return message size
const UINT TS2000_VFOBSIZE	= 14; // Expected return message size
const UINT TS2000_STATUSSIZE	= 18; // Expected return message size

//***** COM PORT PARAMETERS ******************************************
const USHORT TS2000_TIMEOUT	= 1;	// 1 second response timer
const UINT	TS2000_BITRATE	= 9600; // default bit rate
const UINT	TS2000_DATABITS	= 8;
const UINT	TS2000_STOPBITS	= 2;
const UARTPARITY TS2000_PARITY	= UARTPARITY_NONE;

//***** Message State ******************************************
// This type is used to record the request sent to the radio
enum TS2000_MESSAGE {
	TS2000_MESSAGE_NONE,
	TS2000_MESSAGE_RXVFO,
	TS2000_MESSAGE_TXVFO
};

//***** Bands for TS2000 **************************************
// To check we don't try to set the uplink & downlink bands the same.
enum TS2000_BAND {
	TS2000_BAND_H,
	TS2000_BAND_V,
	TS2000_BAND_U,
	TS2000_BAND_L
};

class TS2000 : public RADIO {
private:
	INTERVALTIMER	Timer;		// used for COM timeout
	TS2000_MESSAGE	eMessage;	// expected message type from radio
	BOOL		eSatMode;	// True if using satellite mode
	BOOL		eReady;		// ready for next command
	BOOL		eCheckBand;		// True if first time thru' and we might
								// try to set RX & TX band to same band
								// (not permitted by TS-2000)

	// Rx Params
	LONG		lRxVFOSteps;	// receiver VFO steps
	MODULATION	eRxMode;	// current rx modulation type
	BOOL		eUserHasTunedRx;// TRUE if user changed VFO
	TS2000_BAND	eRxBandInit;	// Initial RX Band

	// Tx Params
	LONG		lTxVFOSteps;	// last VFO steps sent to radio
	LONG		lTxXITSteps;	// sub band tuning adjust (XIT)
	MODULATION	eTxMode;	// current Tx modulation type
	TS2000_BAND	eTxBandInit;	// Initial TX Band
	
	// Com port driver
	COMDRV		Comport;
	
	// Message Buffers
	CHAR	aOutputBuffer[TS2000_MAXMESSAGE];

	UCHAR	aInputBuffer[TS2000_MAXMESSAGE];
	UINT	unByteCounter;	// received bytes in input buffer
	UINT	unMessageSize;  // expected recieved bytes in input buffer
	

	//******************************************************
	// Utilities
	//******************************************************
	
	// Table to convert RADIO::MODULATION type to TS-2000 message param
	static const CHAR aSetModeTable[];

	// Table to convert TS-2000 message param to RADIO::MODULATION type
	static const MODULATION aGetModeTable[];
	
	// Send a message to the radio
	VOID	SendMessage(void);
	VOID	SendMessage(const CHAR *aMessage);	
	VOID	SendMessage(UINT unSize);	
	VOID	SendMessage(const CHAR *aMessage, UINT unSize);
	
	// Convert Frequency to steps
	LONG	FreqtoSteps(DOUBLE dFrequency) {

		//************************************************
		// WARNING: this code has a workaround for MS C7 bug
		// converting double to long. Do NOT change!
		//************************************************
		volatile ULONG ulFrequency = (ULONG) dFrequency;
		LONG lFrequency = (LONG) ulFrequency;
		return ( (lFrequency) + (TS2000_STEPSIZE/2) ) / TS2000_STEPSIZE;
	}

	// Set actual radio frequency
	VOID SetFrequency(LONG	lFrequencySteps,const CHAR* pszSetVFOCmd);

	
	// Process GetVFO response message
	VOID	ProcessVFO(LONG& lFreqSteps, MODULATION& eMode);

	TS2000_BAND FreqToBand(DOUBLE d)
	{
		if (d<100000000.0) {
			return TS2000_BAND_H;
		}
		else {
			if (d<300000000.0) {
				return TS2000_BAND_V;
			}
			else {
				if (d<1000000000.0) {
					return TS2000_BAND_U;
				}
				else {
					return TS2000_BAND_L;
				}
			}
		}
	}
		
	// No copy constructor or assignment allowed
	// Force compile error if attempted
	TS2000(const TS2000&);
	TS2000& operator=(const TS2000&);

public:
	// General Functions
	TS2000(const RADIOPARAMS& Params);
	BOOL	IsReady() { return eReady; }
	VOID	Update();
	~TS2000();

	// Tx Functions
	VOID	SetTxFreq(DOUBLE dFrequency);
	VOID	SetTxMode(MODULATION eMode);

	// Rx Functions
	BOOL	HasUserTunedRx() { return eUserHasTunedRx; }
	VOID	RefreshRxParams();
	DOUBLE	GetRxFreq();
	MODULATION GetRxMode() { return eRxMode; }
	VOID	SetRxFreq(DOUBLE dFrequency);
	VOID	SetRxMode(MODULATION eMode);

};
#endif // end ts2000.hpp
