/****************************************************************************
Date: 9/9/2001
File: ts790.cpp
Description:
	KENWOOD TS-790 radio driver

History:
 	6/14/2001 Initial Version
	6/21/2001 B Version
	7/3/2001 improve SUB tuning
	7/28/2001 add RIT to SUB
	8/1/2001 RIT doesn't work so use XIT on MAIN
	8/2/2001 fix bug in IF decode routine
	8/4/2001 Change TX tuning step to 100Hz 
	8/5/2001 make MAIN update wait an extra second 
	8/17/2001 delay setting SUB until MAIN is set (init problem)
	8/18/2001 take out MAIN update delay
	9/2/2001 make Tx step size programmable in ssb/cw modes
	9/9/2001 add TX and RX on MAIN modes
*****************************************************************************/
#include "ts790.hpp"

// Mode Tables
// Index into table is kenwood mode code, returns MODULATION value
const MODULATION TS790::aGetModeTable[] = {
	MODULATION_NA,	// 0 = invalid
	MODULATION_LSB,	// 1
	MODULATION_USB,	// 2
	MODULATION_CW,	// 3
	MODULATION_FM,	// 4
	MODULATION_NA,	// 5 No AM on TS-790
	MODULATION_NA,	// 6 No FSK on TS-790
	MODULATION_CW	// 7 = CW narrow
};

// Index into table is MODULATION type, returns kenwood mode code
const CHAR TS790::aSetModeTable[] = {
	'0',	// NA
	'1',	// LSB
	'2',	// USB
	'3',	// CW
	'4',	// FM
	'2',	// AM use USB
	'2'	// FSK use USB
};

/****************************************************************************
Function: SetFrequency()				PRIVATE
Description:
	Calculates difference between new frequency and current setting.
	If difference is bigger than step size, commands radio to change
	frequency using mic buttons or direct VFO setting and updates
	current frequency step count.

Parameters:
	LONG	lNewSteps,	// desired frequency step count
	LONG	lStepSize,	// frequency step size
	const CHAR* pszSetVFO	// Set VFO command string

Returns:
*****************************************************************************/
VOID	TS790::SetFrequency(LONG lNewSteps,
			    LONG lStepSize,
			    const CHAR* pszSetVFOCmd) {

	//******************************************************
	// Create "Set VFO" message and Send it out.
	// Put "set VFO" opcode in Buffer. Then convert
	// frequency to a string and put in output buffer.
	// Finally, send message to Serial Link
	//*******************************************************

	CHAR*	pBuffer = aOutputBuffer;
	strcpy(pBuffer, pszSetVFOCmd);
	pBuffer += TS790_OPCODESIZE;

	//************************************************************
	// Maximum frequency that LONG can support is 2GHz
	// This is 10 digits but kenwood format is 11 digits so
	// first write leading zero then iterate over remaining digits
	// converting LONG to a character string. Can't use ltoa()
	// because we need leading zeros.
	//*************************************************************

	*pBuffer++ = '0';	// write leading 0 (10 GHz)

	LONG lFrequency = lNewSteps * lStepSize; // steps to freq
	LONG lDivisor = TS790_1GIGAHERTZ;
	for (INT i=0; i < 10; i++) {
		LONG lDigit = lFrequency / lDivisor; 
		*pBuffer++ = (CHAR) (lDigit + '0');
		lFrequency -= (lDigit * lDivisor);
		lDivisor /= 10;	
	}
	*pBuffer = 0; 	// add nul termination

	SerialLink.Send(aOutputBuffer);

	return;
}


/****************************************************************************
Function: SetMode()					PRIVATE
Description:
	Sends radio the "Set Mode" command to set the modulation type
Parameters:
	Mode
Returns:
*****************************************************************************/
VOID	TS790::SetMode(MODULATION eMode) {

	//********************************************
	// Copy "set mode" opcode to output buffer.
	// Set the kenwood mode code for the desired
	// mode in the output buffer. Then send it out.
	//*********************************************
	strcpy(aOutputBuffer, TS790_MODE);
	aOutputBuffer[TS790_OPCODESIZE] = aSetModeTable[eMode];
	SerialLink.Send(aOutputBuffer, TS790_OPCODESIZE + 1);

	return;
}


/*****************************************************************************
Function: GetRxStepSize()					PRIVATE
Description:
	Gets Rx frequency step size for this modulation mode
Parameters:
	Mode
Returns:
	Step size
*****************************************************************************/
LONG	TS790::GetRxStepSize(MODULATION eMode) {

	LONG lStep;
	if (eMode == MODULATION_FM) {
		lStep = TS790_STEP_FM;
	}
	else {
		lStep = TS790_STEP_SSB;
	}

	return lStep;
}

/*****************************************************************************
Function: GetTxStepSize()					PRIVATE
Description:
	Get Tx frequency step size for this modulation mode
Parameters:
	Mode
Returns:
	Step size
*****************************************************************************/
LONG	TS790::GetTxStepSize(MODULATION eMode) {

	LONG lStep;
	if (eMode == MODULATION_FM) {
		lStep = TS790_STEP_FM;
	}
	else {
		lStep = lTxSSBStepSize;
	}

	return lStep;
}

/*****************************************************************************
Function: SelectTXBand()					PRIVATE
Description:
	Selects BAND (MAIN or SUB) used for TX
Parameters:
Returns:
*****************************************************************************/
VOID	TS790::SelectTXBand() {

	if (eXCVR) {
		// Lock Front Panel
		SerialLink.Send(TS790_LOCKON);

		// Select MAIN BAND
		SerialLink.Send(TS790_MAINBAND);
	}
}

/*****************************************************************************
Function: SelectRXBand()					PRIVATE
Description:
	Selects BAND (MAIN or SUB) used for RX
Parameters:
Returns:
*****************************************************************************/
VOID	TS790::SelectRXBand() {

	if (eXCVR) {
		// Select SUB BAND
		SerialLink.Send(TS790_SUBBAND);

		// Un-lock Front Panel
		SerialLink.Send(TS790_LOCKOFF);
	}

}


/****************************************************************************
Function: TS790()
Description: constructor
Parameters:
	RADIOPARAMS structure
Returns:
*****************************************************************************/
TS790::TS790(const RADIOPARAMS& Params) :
	
	// Initialize
	RADIO(),
	eXCVR(Params.eTransceive),
	eReady(BOOL_TRUE),
	eHoldTuning(BOOL_FALSE),
	eMainSet(BOOL_FALSE),
	dSaveFrequency(0.0),

	pszRXVFO(0),
	lRxStepSize(TS790_STEP_SSB),
	lRxSteps(0),
	eRxMode(MODULATION_NA),
	eUserHasTunedRx(BOOL_FALSE),

	lTxVFOSteps(0),
	lTxXITSteps(0),
	lTxActualSteps(0),
	eTxMode(MODULATION_NA),

	SerialLink((CHAR*) aInputBuffer,
		   TS790_MAXMESSAGE,
		   TS790_ENDOFMESSAGE,
		   TS790_TIMEOUT)
	
	{ // end init list

	lTxSSBStepSize = Params.unParam1 ? Params.unParam1 : 100;
	lTxStepSize = lTxSSBStepSize;

	//********************************************************
	// Open Serial Link and check communications with radio
	// If fails, return with Operational flag FALSE
	//********************************************************
	if (SerialLink.Open(
			Params.unPort,
			TS790_BITRATE,
			TS790_DATABITS,
			TS790_PARITY,
			TS790_STOPBITS,
			Params.unIRQ,
			Params.Address) == BOOL_FALSE) {

		RADIO::eOperational = BOOL_FALSE;
		return;					// failed
	}

	// Check communications
	SerialLink.Send(TS790_TESTMESSAGE);// Send test msg, wait for response
	do {
		if ( SerialLink.TimeOut() ) {
			SerialLink.Close(); 		// failed
			RADIO::eOperational = BOOL_FALSE;
			return;
		}
	} while(SerialLink.Receive() == 0);

	
	//***** Set up MAIN VFO ************************************
	SerialLink.Send(TS790_MAINBAND);
	SerialLink.Send(TS790_STEPOFF); // turn off step tuning
	SerialLink.Send(TS790_SELECTVFOB); // Use VFO B

	//***** Set up RX VFO as SUB if in XCVR mode *********************
	if (eXCVR) {
		SerialLink.Send(TS790_SUBBAND); // use SUB BAND for RX
		SerialLink.Send(TS790_STEPOFF); // turn off step tuning
		SerialLink.Send(TS790_SELECTVFOA); // Use VFO A for RX
		pszRXVFO = (const CHAR*) TS790_VFOA;	// VFO A if in SUB
	}
	else {
		// otherwise use MAIN
		pszRXVFO = (const CHAR*) TS790_VFOB;	// use VFO B in MAIN
		eMainSet = BOOL_TRUE;	// used in XCVR mode only
	}
	// All OK!
}

/****************************************************************************
Function: 	~TS790()
Description:	destructor
Parameters:
Returns:
*****************************************************************************/
TS790::~TS790() {
	SerialLink.Close();
}

/****************************************************************************
Function: Update()
Description:
	This function implements the processing for getting the
	receiver status. Processing is started with the RefreshRxParams()
	function which send a get information (i.e. IF) command to the radio
	and clears the Ready flag. When the IF response is received, we
	process the response and then send the get VFO A frequency command
	to the Radio. When we receive this response, we process this message
	and set the Ready flag TRUE.

	The information that we get from the receiver is the current
	modulation mode and the current frequency. 

Parameters:
Returns:
*****************************************************************************/
VOID	TS790::Update() {

	//***** If Ready flag is set, nothing to do ****************
    	if (eReady) {
		return;
    	}

	//**************************************************************
	// Check serial link for a received message
	// Value returned is byte count of message or 0 if not available
	// Message is returned in InputBuffer
	// If Message, received, decode and process it
	//*************************************************************

	UINT	unCharCount = SerialLink.Receive();

	if ( unCharCount == 0 ) {

		// No Message received, check for timeout
		if (SerialLink.TimeOut() ) {
			// Timeout!
			SerialLink.Reset();	// clear buffers
			RefreshRxParams();	// try again
		}
		return;
	}

	// Check for valid message received (INFORMATION response msg)
	CHAR*	pMsg = aInputBuffer;

	if ( ((INT) pMsg[0] + pMsg[1]) != TS790_MSGIF_CODE) {
		// Got an unexpected or error response, just ignore
		return;
	}

	//**************************************************
	// Process INFORMATION (IF response) message
	//
	// If in XCVR mode and RIT status parameter says MAIN is
	// selected (RIT != NONE.) then so, do the XIT processing
	//
	// Else do the "normal" receive processing 
	//
	//*************************************************


	if (eXCVR && (pMsg[TS790_MSGIF_RITOFFSET] != TS790_MSGIF_NONE)) {

		//*************************************************
		// Do XIT processing
		//
		// Get display frequency, convert to steps
		// and subtract from TxVFOSteps to create XIT offset.
		//*************************************************

		eHoldTuning = BOOL_TRUE;  // stop doppler tuning


		// Note: I am cheating here. Instead of copying frequency to
		// a buffer and adding a terminating 0, I just jam a 0 into
		// the buffer in the next param which is the STEP FREQUENCY.
		pMsg[TS790_MSGIF_STEPOFFSET] = 0;
		LONG lNewSteps = FreqtoSteps(
					atol(&pMsg[TS790_MSGIF_FREQOFFSET]),
					lTxStepSize);

		lTxXITSteps = lNewSteps - lTxVFOSteps;
	}
	else {
		//*************************************************
		// not in XCVR mode OR SUB selected in XCVR mode
		// Do normal receive processing
		//
		// Get "mode" parameter from message
		// and convert to MODULATION type via table.
		// If mode is FM, TS-790 automatically
		// sets MIC Button step size to 100 Hz.
		//
		// Get frequency, convert to steps and save.
		// Check for manual tuning by user
		//
		//*************************************************
		eHoldTuning = BOOL_FALSE; 	// operate normally 

		eRxMode = aGetModeTable[(pMsg[TS790_MSGIF_MODEOFFSET]-'0')];
		lRxStepSize = GetRxStepSize(eRxMode);

		// Note: I am cheating here. Instead of copying frequency to
		// a buffer and adding a terminating 0, I just jam a 0 into
		// the buffer in the next param which is the STEP FREQUENCY.
		pMsg[TS790_MSGIF_STEPOFFSET] = 0;
		LONG lNewSteps = FreqtoSteps(
					atol(&pMsg[TS790_MSGIF_FREQOFFSET]),
					lRxStepSize);

		// Check to see if use has tuned dial 
		if (lNewSteps != lRxSteps) {
			eUserHasTunedRx = BOOL_TRUE;
			lRxSteps = lNewSteps;
		}
	}
	eReady = BOOL_TRUE;	// done, set ready flag

    	return;
}

/****************************************************************************
Function: SetTxFreq()
Description:
	Sets transmitter to new frequency
Parameters:
	Frequency
Returns:
*****************************************************************************/
VOID	TS790::SetTxFreq(DOUBLE dNewFrequency) {
	
	// Convert new frequency to steps and save
	lTxVFOSteps = FreqtoSteps( dNewFrequency, lTxStepSize );

	// If user is adjusting XIT, don't update the radio
	if (eHoldTuning) {
		return;
	}

	// Add XIT to virtual VFO to get new radio frequency
	LONG lNewSteps = lTxVFOSteps + lTxXITSteps;

	// Check to see if actual radio freq has changed
	if (lNewSteps == lTxActualSteps) { 
		return; 		// no difference, do nothing
	}
	
	// New Frequency is different, save new step count
	lTxActualSteps = lNewSteps;

	// Select BAND used for TX
	SelectTXBand();

	// Set VFO to new frequency (use VFO B)
	SetFrequency(lTxActualSteps, lTxStepSize, TS790_VFOB);

	// Re-Select BAND used for RX
	SelectRXBand();
	
	// if this is the first time in XCVR mode, we need to set the SUB
	if (!eMainSet) {
		eMainSet = BOOL_TRUE;
		SetRxFreq(dSaveFrequency);
	}

	return;
}

/****************************************************************************
Function: SetTxMode()
Description:
	Set transmitter Mode type
Parameters:
	Mode
Returns:
*****************************************************************************/
VOID	TS790::SetTxMode(MODULATION eMode) {
	
	// Check for changed and valid modulation mode
	if ( (eTxMode == eMode) ||
	     (eMode == MODULATION_NA) ) {
		return;				// do nothing
	}

	eTxMode = eMode;		// Save New mode
	lTxStepSize = GetTxStepSize(eMode); // Adjust Tx Step Size
	
	// Select BAND used for TX
	SelectTXBand();

	// Set New TX Mode
	SetMode(eMode);

	// Re-Select BAND used for RX
	SelectRXBand();

	return;
}

/****************************************************************************
Function: RefreshRxParams()
Description:
	Initiate getting radio info (Rx Freq, Rx Modulation)
Parameters:
Returns:
*****************************************************************************/
VOID	TS790::RefreshRxParams() {

	eReady = BOOL_FALSE;
	SerialLink.Send(TS790_INFO);  	// Read info
	return;
}

/****************************************************************************
Function: GetRxFreq()
Description:
	Gets current rx frequency
Parameters:
Returns:
	receiver frequency
*****************************************************************************/
DOUBLE	TS790::GetRxFreq() {
	eUserHasTunedRx = BOOL_FALSE;
	return ((DOUBLE) lRxSteps) * ((DOUBLE) lRxStepSize);
}

/****************************************************************************
Function: SetRxFreq()
Description:
	Sets Receiver to new frequency
Parameters:
	Frequency
Returns:
*****************************************************************************/
VOID	TS790::SetRxFreq(DOUBLE dFrequency) {

	// Delay setting SUB if MAIN not set yet in XCVR mode
	if (!eMainSet) {
		dSaveFrequency = dFrequency; // remember desired SUB freq
		return;
	}

	// If in XIT capture mode, don't tune SUB
	if (eHoldTuning) {
		eUserHasTunedRx = BOOL_TRUE;
		return;
	}

	// Reset user tuning indicator flag
	eUserHasTunedRx = BOOL_FALSE;

	// Convert new frequency to steps
	LONG lNewSteps = FreqtoSteps(dFrequency, lRxStepSize);

	// Check to see if actual radio freq needs to be changed
	if (lNewSteps == lRxSteps) {
		return; 		// no difference, do nothing
	}
	
	// New Frequency is different, Save new step count
	lRxSteps = lNewSteps;

	// Set VFO to new frequency
	SetFrequency(lRxSteps, lRxStepSize, pszRXVFO);

	return;
}

/****************************************************************************
Function: SetRxMode()
Description:
	Set receive modulation mode type
Parameters:
	Mode
Returns:
*****************************************************************************/
VOID	TS790::SetRxMode(MODULATION eMode) {
	
	// Check for changed and valid modulation mode
	if ( (eRxMode == eMode) ||
	     (eMode == MODULATION_NA) ) {
		return;				// do nothing
	}

	eRxMode = eMode; 		// Save new mode
	SetMode(eMode); 		// Send command to radio
	lRxStepSize = GetRxStepSize(eMode);// save new step size

	return;
}

// end ts790.cpp
