/****************************************************************************
Copyright 1997 Anthony Monteiro

Date: 11/19/97
File: xpdrbase.hpp
Description:	TRANSPONDER ABSTRACT BASE CLASS

	This abstract class is the base class for all transponders. 

	To create a transponder object, use the NewInstance() function in
	this class and pass the transponder parameters. NewInstance() allocates
	memory and creates the proper type of object (essentially a virtual
	constructor.) To delete transponder objects, use the usual delete().
	
	public:
		static TRANSPONDER* 
			NewInstance(const TRANSPONDERPARAMS& Params)

		DOUBLE GetInputFreq() const
		DOUBLE GetOutputFreq() const

		MODULATION GetInputMode() const
		MODULATION GetOutputMode() const

		virtual VOID SetInputFreq(Double dFreq)	= 0
		virtual VOID SetOutputFreq(DOUBLE dFreq) = 0

		virtual VOID SetInputMode(MODULATION eMode) = 0;
		virtual VOID SetOutputMode(MODULATION eMode) = 0;

		virtual ~TRANSPONDER()

History:
	11/19/97 Version 1.0
*****************************************************************************/
#ifndef XPDRBASE_HPP
#define XPDRBASE_HPP

#include "loctypes.h"
#include "modulate.hpp"

enum	TRANSPONDERTYPE {
	TRANSPONDERTYPE_NULL = 0,	// no transponder
	TRANSPONDERTYPE_FIXED,		// fixed up and down links
	TRANSPONDERTYPE_NONINVERTING,	// linear, non inverting
	TRANSPONDERTYPE_INVERTING,	// linear, inverting
};

// Transponder parameters define the type and behavior of the transponder
// Not all transponder types need all parameters (they can be ignored)
// "dParam" is a transponder specific parameter.
//  - For linear transponders, it is the translation parameter.
//  - For fixed transponders, it is the actual uplink frequency
struct	TRANSPONDERPARAMS {
	TRANSPONDERTYPE	eType;	
	DOUBLE		dOutFreq;	// downlink or beacon frequency
	DOUBLE		dParam;		// transponder parameter
	MODULATION 	eOutMode;	// downlink modulation type
	MODULATION 	eInMode;	// uplink modulation type
};

class TRANSPONDER {
private:
	TRANSPONDER(const TRANSPONDER&);	// no copy or = allowed
	TRANSPONDER& operator=(const TRANSPONDER&);

protected:
	DOUBLE		dInputFreq;
	DOUBLE		dOutputFreq;

	MODULATION	eInputMode;
	MODULATION	eOutputMode;

	TRANSPONDER() {}

public:
	static	TRANSPONDER* NewInstance(const TRANSPONDERPARAMS& Params);

	DOUBLE		GetInputFreq()	const { return dInputFreq; }
	DOUBLE		GetOutputFreq()	const { return dOutputFreq; }

	MODULATION 	GetInputMode()	const { return eInputMode; }
	MODULATION 	GetOutputMode() const { return eOutputMode; }

	virtual VOID	SetInputFreq(DOUBLE dFreq)	= 0;
	virtual VOID	SetOutputFreq(DOUBLE dFreq) 	= 0;

	virtual VOID	SetInputMode(MODULATION eMode) 	= 0;
	virtual VOID	SetOutputMode(MODULATION eMode) = 0;

	virtual ~TRANSPONDER() {}
};
#endif // xpdrbase.hpp
